# -*- coding: utf-8 -*-

# Copyright (C) 2012-2013 Avencall
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>


import sys
import logging
from logging.handlers import SysLogHandler

from xivo_agid import agid
from xivo_agid.modules import *
from xivo import daemonize

PIDFILE = "/var/run/xivo-agid.pid"
SYSLOG_NAME = 'xivo-agid'


def main():
    logging.basicConfig(level=logging.DEBUG)
    sysloghandler = SysLogHandler("/dev/log", SysLogHandler.LOG_DAEMON)
    sysloghandler.setFormatter(logging.Formatter("%s[%%(process)d]: %%(message)s" % SYSLOG_NAME))
    logging.getLogger('').addHandler(sysloghandler)

    if '-f' not in sys.argv:
        daemonize.daemonize()

    daemonize.lock_pidfile_or_die(PIDFILE)
    try:
        if '-d' not in sys.argv:
            logging.getLogger('').setLevel(logging.INFO)

        agid.init()
        agid.run()
    finally:
        daemonize.unlock_pidfile(PIDFILE)


if __name__ == '__main__':
    main()
